@@ -438,6 +438,7 @@ export class ASRouterAdminInner extends React.PureComponent {
       "spotlight",
       "cfr_doorhanger",
       "feature_callout",
+      "pb_newtab",
     ].includes(msg.template);
 
     let itemClassName = "message-item";
@@ -1216,7 +1217,7 @@ export class ASRouterAdminInner extends React.PureComponent {
   }
 
   onChangeAttributionParameters(event) {
-    const { eventName, value } = event.target;
+    const { name: eventName, value } = event.target;
 
     this.setState(({ attributionParameters }) => {
       const updatedParameters = { ...attributionParameters };
