@@ -1406,9 +1406,12 @@ async function triggerCapture(browser, submitButtonSelector, fillSelectors) {
  *        Region to assign before running the test
  * @param {Array} patterns.expectedResult
  *        The expected result of this heuristic test. See below for detailed explanation
+ * @param {Function} patterns.onTestStart
+ *        Function that is executed before the test starts. This runs after the form
+ *        field has been focused.
  * @param {Function} patterns.onTestComplete
- *        Function that is executed when the test is complete. This can be used by the test
- *        to verify the status after running the test.
+ *        Function that is executed when the test is complete, but before the tab is closed.
+ *        This can be used by the test to verify the status after running the test.
  *
  * @param {string} patterns.autofillTrigger
  *        The selector to find the element to trigger the autocomplete popup.
@@ -1542,8 +1545,14 @@ async function add_heuristic_tests(
       info(`Focus on each field in the test document`);
       const contexts =
         browser.browsingContext.getAllBrowsingContextsInSubtree();
+
+      // This is a workaround for when we set focus on elements across iframes (in the previous step).
+      // The popup is not refreshed, and consequently, it does not receive key events needed to trigger
+      // the autocomplete popup.
+      const sleepAfterFocus = contexts.length > 1;
+
       for (const context of contexts) {
-        await SpecialPowers.spawn(context, [], async function () {
+        await SpecialPowers.spawn(context, [], async () => {
           const elements = Array.from(
             content.document.querySelectorAll("input, select")
           );
@@ -1556,16 +1565,16 @@ async function add_heuristic_tests(
 
         try {
           await BrowserTestUtils.synthesizeKey("VK_ESCAPE", {}, context);
+          if (sleepAfterFocus) {
+            await sleep();
+          }
         } catch (e) {
           // Error occurs when sending a key event to an invisible iframe, ignore the error.
         }
       }
 
-      // This is a workaround for when we set focus on elements across iframes (in the previous step).
-      // The popup is not refreshed, and consequently, it does not receive key events needed to trigger
-      // the autocomplete popup.
-      if (contexts.length > 1) {
-        await sleep();
+      if (testPattern.onTestStart) {
+        await testPattern.onTestStart();
       }
 
       info(`Waiting for expected section count`);
@@ -1618,11 +1627,11 @@ async function add_heuristic_tests(
         verifyCaptureRecord(guid, testPattern.captureExpectedRecord);
         await removeAllRecords();
       }
-    });
 
-    if (testPattern.onTestComplete) {
-      await testPattern.onTestComplete();
-    }
+      if (testPattern.onTestComplete) {
+        await testPattern.onTestComplete();
+      }
+    });
 
     if (testPattern.profile) {
       await removeAllRecords();
