@@ -4,7 +4,9 @@
 
 "use strict";
 
-const sharedJestConfig = require(`${__dirname}/../shared/test-helpers/shared-jest.config`);
+const sharedJestConfig = require(
+  `${__dirname}/../shared/test-helpers/shared-jest.config`
+);
 
 module.exports = {
   testEnvironment: "jsdom",
