@@ -373,7 +373,7 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
     }
   );
   await waitForPaused(dbg);
-  assertSourceIcon(dbg, "react-component-module.js", "react");
+  assertSourceIcon(dbg, "react-component-module.js", "javascript");
   await resume(dbg);
   await onResumed;
 
@@ -451,7 +451,11 @@ add_task(async function testSourceTreeWithWebExtensionContentScript() {
   let dbg = await initDebugger("doc-content-script-sources.html");
   // Let some time for unexpected source to appear
   await wait(1000);
-  await waitForSourcesInSourceTree(dbg, []);
+  // There is no content script, but still html pages inline sources
+  await waitForSourcesInSourceTree(dbg, [
+    "doc-content-script-sources.html",
+    "doc-strict.html",
+  ]);
   await dbg.toolbox.closeToolbox();
 
   const toolbox = await openToolboxForTab(gBrowser.selectedTab, "jsdebugger");
@@ -463,14 +467,19 @@ add_task(async function testSourceTreeWithWebExtensionContentScript() {
     isChecked: false,
   });
 
-  await waitForSourcesInSourceTree(dbg, ["content_script.js"]);
+  await waitForSourcesInSourceTree(dbg, [
+    "doc-content-script-sources.html",
+    "doc-strict.html",
+    "content_script.js",
+  ]);
   await selectSource(dbg, "content_script.js");
   ok(
     findElementWithSelector(dbg, ".sources-list .focused"),
     "Source is focused"
   );
 
-  const contentScriptGroupItem = findSourceNodeWithText(
+  // Note that the thread item also contains the extension name
+  const contentScriptGroupItem = findSourceTreeGroupByName(
     dbg,
     "Test content script extension"
   );
