@@ -235,14 +235,11 @@ class Grid;
 class OwningTrustedHTMLOrNullIsEmptyString;
 class TrustedHTML;
 class TrustedHTMLOrNullIsEmptyString;
+class TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString;
 
 // IID for the dom::Element interface
-#define NS_ELEMENT_IID                               \
-  {                                                  \
-    0xc67ed254, 0xfd3b, 0x4b10, {                    \
-      0x96, 0xa2, 0xc5, 0x8b, 0x7b, 0x64, 0x97, 0xd1 \
-    }                                                \
-  }
+#define NS_ELEMENT_IID \
+  {0xc67ed254, 0xfd3b, 0x4b10, {0x96, 0xa2, 0xc5, 0x8b, 0x7b, 0x64, 0x97, 0xd1}}
 
 #define REFLECT_NULLABLE_DOMSTRING_ATTR(method, attr)            \
   void Get##method(nsAString& aValue) const {                    \
@@ -266,6 +263,17 @@ class TrustedHTMLOrNullIsEmptyString;
     ExplicitlySetAttrElement(nsGkAtoms::attr, aElement); \
   }
 
+#define REFLECT_NULLABLE_ELEMENTS_ATTR(method, attr)                        \
+  void Get##method(bool* aUseCachedValue,                                   \
+                   Nullable<nsTArray<RefPtr<Element>>>& aElements) {        \
+    GetAttrAssociatedElements(nsGkAtoms::attr, aUseCachedValue, aElements); \
+  }                                                                         \
+                                                                            \
+  void Set##method(                                                         \
+      const Nullable<Sequence<OwningNonNull<Element>>>& aElements) {        \
+    ExplicitlySetAttrElements(nsGkAtoms::attr, aElements);                  \
+  }
+
 // TODO(keithamus): Reference the spec link once merged.
 // https://github.com/whatwg/html/pull/9841/files#diff-41cf6794ba4200b839c53531555f0f3998df4cbb01a4d5cb0b94e3ca5e23947dR86024
 enum class InvokeAction : uint8_t {
@@ -705,21 +713,28 @@ class Element : public FragmentOrElement {
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaColIndex, aria_colindex)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaColIndexText, aria_colindextext)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaColSpan, aria_colspan)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaControlsElements, aria_controls)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaCurrent, aria_current)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaDescribedByElements, aria_describedby)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaDescription, aria_description)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaDetailsElements, aria_details)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaDisabled, aria_disabled)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaErrorMessageElements, aria_errormessage)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaExpanded, aria_expanded)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaFlowToElements, aria_flowto)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaHasPopup, aria_haspopup)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaHidden, aria_hidden)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaInvalid, aria_invalid)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaKeyShortcuts, aria_keyshortcuts)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaLabel, aria_label)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaLabelledByElements, aria_labelledby)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaLevel, aria_level)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaLive, aria_live)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaModal, aria_modal)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaMultiLine, aria_multiline)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaMultiSelectable, aria_multiselectable)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaOrientation, aria_orientation)
+  REFLECT_NULLABLE_ELEMENTS_ATTR(AriaOwnsElements, aria_owns)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaPlaceholder, aria_placeholder)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaPosInSet, aria_posinset)
   REFLECT_NULLABLE_DOMSTRING_ATTR(AriaPressed, aria_pressed)
@@ -1171,6 +1186,10 @@ class Element : public FragmentOrElement {
                                       const MappedAttributeEntry* const aMaps[],
                                       uint32_t aMapCount);
 
+  bool HasSharedRoot(const Element* aElement) const;
+
+  Element* GetElementByIdInDocOrSubtree(nsAtom* aID) const;
+
  protected:
   inline bool GetAttr(const nsAtom* aName, DOMString& aResult) const {
     MOZ_ASSERT(aResult.IsEmpty(), "Should have empty string coming in");
@@ -1207,6 +1226,9 @@ class Element : public FragmentOrElement {
     return false;
   }
 
+  // Note, this does not notify about the removal.
+  void ClearAttributes() { mAttrs.Clear(); }
+
   void GetTagName(nsAString& aTagName) const { aTagName = NodeName(); }
   void GetId(nsAString& aId) const { GetAttr(nsGkAtoms::id, aId); }
   void GetId(DOMString& aId) const { GetAttr(nsGkAtoms::id, aId); }
@@ -1250,6 +1272,22 @@ class Element : public FragmentOrElement {
                     ErrorResult& aError) {
     SetAttribute(aName, aValue, nullptr, aError);
   }
+
+  MOZ_CAN_RUN_SCRIPT void SetAttribute(
+      const nsAString& aName,
+      const TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString& aValue,
+      nsIPrincipal* aTriggeringPrincipal, ErrorResult& aError);
+  MOZ_CAN_RUN_SCRIPT void SetAttributeNS(
+      const nsAString& aNamespaceURI, const nsAString& aLocalName,
+      const TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString& aValue,
+      nsIPrincipal* aTriggeringPrincipal, ErrorResult& aError);
+  MOZ_CAN_RUN_SCRIPT void SetAttribute(
+      const nsAString& aName,
+      const TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString& aValue,
+      ErrorResult& aError) {
+    SetAttribute(aName, aValue, nullptr, aError);
+  }
+
   /**
    * This method creates a principal that subsumes this element's NodePrincipal
    * and which has flags set for elevated permissions that devtools needs to
@@ -1288,14 +1326,21 @@ class Element : public FragmentOrElement {
    * https://html.spec.whatwg.org/multipage/common-dom-interfaces.html#attr-associated-element
    */
   Element* GetAttrAssociatedElement(nsAtom* aAttr) const;
+  void GetAttrAssociatedElements(
+      nsAtom* aAttr, bool* aUseCachedValue,
+      Nullable<nsTArray<RefPtr<Element>>>& aElements);
 
   /**
    * Sets an attribute element for the given attribute.
    * https://html.spec.whatwg.org/multipage/common-dom-interfaces.html#explicitly-set-attr-element
    */
   void ExplicitlySetAttrElement(nsAtom* aAttr, Element* aElement);
+  void ExplicitlySetAttrElements(
+      nsAtom* aAttr,
+      const Nullable<Sequence<OwningNonNull<Element>>>& aElements);
 
   void ClearExplicitlySetAttrElement(nsAtom*);
+  void ClearExplicitlySetAttrElements(nsAtom*);
 
   /**
    * Gets the attribute element for the given attribute.
@@ -1307,6 +1352,16 @@ class Element : public FragmentOrElement {
    */
   Element* GetExplicitlySetAttrElement(nsAtom* aAttr) const;
 
+  /**
+   * Gets the attribute elements for the given attribute. Unlike
+   * GetAttrAssociatedElements, this returns an uncached array of explicitly set
+   * elements without checking if they are a descendant of any of this element's
+   * shadow-including ancestors. It also does not attempt to retrieve elements
+   * using the ids set in the content attribute.
+   */
+  void GetExplicitlySetAttrElements(nsAtom* aAttr,
+                                    nsTArray<Element*>& aElements) const;
+
   PseudoStyleType GetPseudoElementType() const {
     nsresult rv = NS_OK;
     auto raw = GetProperty(nsGkAtoms::pseudoProperty, &rv);
@@ -1338,13 +1393,13 @@ class Element : public FragmentOrElement {
 
   /**
    * Get an editor which handles user inputs when this element has focus.
-   * If this is a text control, return a TextEditor if it's already created.
-   * Otherwise, return nullptr.
-   * If this is not a text control but this is editable, return
-   * HTMLEditor which should've already been created.
+   * If this is a text control, return a TextEditor if it's already created and
+   * it's not in the design mode.
+   * If this is editable, return HTMLEditor which should've already been
+   * created.
    * Otherwise, return nullptr.
    */
-  EditorBase* GetEditorWithoutCreation() const;
+  EditorBase* GetExtantEditor() const;
 
  private:
   /**
@@ -1577,10 +1632,6 @@ class Element : public FragmentOrElement {
   void GetAnimationsWithoutFlush(const GetAnimationsOptions& aOptions,
                                  nsTArray<RefPtr<Animation>>& aAnimations);
 
-  static void GetAnimationsUnsorted(Element* aElement,
-                                    PseudoStyleType aPseudoType,
-                                    nsTArray<RefPtr<Animation>>& aAnimations);
-
   void CloneAnimationsFrom(const Element& aOther);
 
   virtual void GetInnerHTML(nsAString& aInnerHTML, OOMReporter& aError);
@@ -1603,8 +1654,12 @@ class Element : public FragmentOrElement {
                                    nsIPrincipal* aSubjectPrincipal,
                                    ErrorResult& aError);
 
-  void GetOuterHTML(nsAString& aOuterHTML);
-  void SetOuterHTML(const nsAString& aOuterHTML, ErrorResult& aError);
+  // @param aOuterHTML will always be of type `NullIsEmptyString`.
+  void GetOuterHTML(OwningTrustedHTMLOrNullIsEmptyString& aOuterHTML);
+
+  MOZ_CAN_RUN_SCRIPT void SetOuterHTML(
+      const TrustedHTMLOrNullIsEmptyString& aOuterHTML, ErrorResult& aError);
+
   MOZ_CAN_RUN_SCRIPT void InsertAdjacentHTML(
       const nsAString& aPosition,
       const TrustedHTMLOrString& aTrustedHTMLOrString, ErrorResult& aError);
@@ -1738,6 +1793,8 @@ class Element : public FragmentOrElement {
 
   nsINode* GetScopeChainParent() const override;
 
+  JSObject* WrapNode(JSContext*, JS::Handle<JSObject*> aGivenProto) override;
+
   /**
    * Locate a TextEditor rooted at this content node, if there is one.
    */
@@ -1848,7 +1905,7 @@ class Element : public FragmentOrElement {
    * Get the pseudo element for this pseudo request (i.e. PseudoStyleType and
    * its function parameter, if any).
    */
-  const Element* GetPseudoElement(const PseudoStyleRequest&) const;
+  Element* GetPseudoElement(const PseudoStyleRequest&) const;
 
   ReferrerPolicy GetReferrerPolicyAsEnum() const;
   ReferrerPolicy ReferrerPolicyFromAttr(const nsAttrValue* aValue) const;
@@ -2196,7 +2253,8 @@ class Element : public FragmentOrElement {
   virtual bool Translate() const;
 
   MOZ_CAN_RUN_SCRIPT
-  virtual void SetHTMLUnsafe(const nsAString& aHTML);
+  virtual void SetHTMLUnsafe(const TrustedHTMLOrString& aHTML,
+                             ErrorResult& aError);
 
  protected:
   enum class ReparseAttributes { No, Yes };
@@ -2344,45 +2402,37 @@ inline mozilla::dom::Element* nsINode::GetNextElementSibling() const {
  * Macros to implement Clone(). _elementName is the class for which to implement
  * Clone.
  */
-#define NS_IMPL_ELEMENT_CLONE(_elementName)                         \
+#define NS_IMPL_ELEMENT_CLONE(_elementName, ...)                    \
   nsresult _elementName::Clone(mozilla::dom::NodeInfo* aNodeInfo,   \
                                nsINode** aResult) const {           \
     *aResult = nullptr;                                             \
-    RefPtr<mozilla::dom::NodeInfo> ni(aNodeInfo);                   \
-    auto* nim = ni->NodeInfoManager();                              \
-    RefPtr<_elementName> it = new (nim) _elementName(ni.forget());  \
+    RefPtr<_elementName> it = new (aNodeInfo->NodeInfoManager())    \
+        _elementName(do_AddRef(aNodeInfo), ##__VA_ARGS__);          \
     nsresult rv = const_cast<_elementName*>(this)->CopyInnerTo(it); \
     if (NS_SUCCEEDED(rv)) {                                         \
       it.forget(aResult);                                           \
     }                                                               \
-                                                                    \
     return rv;                                                      \
   }
 
-#define EXPAND(...) __VA_ARGS__
-#define NS_IMPL_ELEMENT_CLONE_WITH_INIT_HELPER(_elementName, extra_args_) \
-  nsresult _elementName::Clone(mozilla::dom::NodeInfo* aNodeInfo,         \
-                               nsINode** aResult) const {                 \
-    *aResult = nullptr;                                                   \
-    RefPtr<mozilla::dom::NodeInfo> ni(aNodeInfo);                         \
-    auto* nim = ni->NodeInfoManager();                                    \
-    RefPtr<_elementName> it =                                             \
-        new (nim) _elementName(ni.forget() EXPAND extra_args_);           \
-    nsresult rv = it->Init();                                             \
-    nsresult rv2 = const_cast<_elementName*>(this)->CopyInnerTo(it);      \
-    if (NS_FAILED(rv2)) {                                                 \
-      rv = rv2;                                                           \
-    }                                                                     \
-    if (NS_SUCCEEDED(rv)) {                                               \
-      it.forget(aResult);                                                 \
-    }                                                                     \
-                                                                          \
-    return rv;                                                            \
+#define NS_IMPL_ELEMENT_CLONE_WITH_INIT(_elementName, ...)           \
+  nsresult _elementName::Clone(mozilla::dom::NodeInfo* aNodeInfo,    \
+                               nsINode** aResult) const {            \
+    *aResult = nullptr;                                              \
+    RefPtr<_elementName> it = new (aNodeInfo->NodeInfoManager())     \
+        _elementName(do_AddRef(aNodeInfo), ##__VA_ARGS__);           \
+    nsresult rv = it->Init();                                        \
+    nsresult rv2 = const_cast<_elementName*>(this)->CopyInnerTo(it); \
+    if (NS_FAILED(rv2)) {                                            \
+      rv = rv2;                                                      \
+    }                                                                \
+    if (NS_SUCCEEDED(rv)) {                                          \
+      it.forget(aResult);                                            \
+    }                                                                \
+    return rv;                                                       \
   }
 
-#define NS_IMPL_ELEMENT_CLONE_WITH_INIT(_elementName) \
-  NS_IMPL_ELEMENT_CLONE_WITH_INIT_HELPER(_elementName, ())
 #define NS_IMPL_ELEMENT_CLONE_WITH_INIT_AND_PARSER(_elementName) \
-  NS_IMPL_ELEMENT_CLONE_WITH_INIT_HELPER(_elementName, (, NOT_FROM_PARSER))
+  NS_IMPL_ELEMENT_CLONE_WITH_INIT(_elementName, NOT_FROM_PARSER)
 
 #endif  // mozilla_dom_Element_h__
