@@ -111,7 +111,7 @@ class CustomElementRegistry;
 class DataTransfer;
 class DocGroup;
 class External;
-class Function;
+class FunctionOrTrustedScriptOrString;
 class Gamepad;
 class ContentMediaController;
 enum class ImageBitmapFormat : uint8_t;
@@ -335,6 +335,9 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   void Freeze(bool aIncludeSubWindows = true);
   void Thaw(bool aIncludeSubWindows = true);
   virtual bool IsFrozen() const override;
+  virtual bool HasActiveIndexedDBDatabases() override;
+  virtual bool HasActivePeerConnections() override;
+  virtual bool HasOpenWebSockets() const override;
   void SyncStateFromParentWindow();
 
   // Called on the current inner window of a browsing context when its
@@ -716,31 +719,22 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
                       mozilla::ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT
-  int32_t SetTimeout(JSContext* aCx, mozilla::dom::Function& aFunction,
-                     int32_t aTimeout,
-                     const mozilla::dom::Sequence<JS::Value>& aArguments,
-                     mozilla::ErrorResult& aError);
-
-  MOZ_CAN_RUN_SCRIPT
-  int32_t SetTimeout(JSContext* aCx, const nsAString& aHandler,
-                     int32_t aTimeout,
-                     const mozilla::dom::Sequence<JS::Value>& /* unused */,
-                     mozilla::ErrorResult& aError);
+  int32_t SetTimeout(
+      JSContext* aCx,
+      const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
+      int32_t aTimeout, const mozilla::dom::Sequence<JS::Value>& /* unused */,
+      mozilla::ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT
   void ClearTimeout(int32_t aHandle);
 
   MOZ_CAN_RUN_SCRIPT
-  int32_t SetInterval(JSContext* aCx, mozilla::dom::Function& aFunction,
-                      const int32_t aTimeout,
-                      const mozilla::dom::Sequence<JS::Value>& aArguments,
-                      mozilla::ErrorResult& aError);
-
-  MOZ_CAN_RUN_SCRIPT
-  int32_t SetInterval(JSContext* aCx, const nsAString& aHandler,
-                      const int32_t aTimeout,
-                      const mozilla::dom::Sequence<JS::Value>& /* unused */,
-                      mozilla::ErrorResult& aError);
+  int32_t SetInterval(
+      JSContext* aCx,
+      const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
+      const int32_t aTimeout,
+      const mozilla::dom::Sequence<JS::Value>& /* unused */,
+      mozilla::ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT
   void ClearInterval(int32_t aHandle);
@@ -1078,19 +1072,14 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   // |interval| is in milliseconds.
   MOZ_CAN_RUN_SCRIPT
   int32_t SetTimeoutOrInterval(
-      JSContext* aCx, mozilla::dom::Function& aFunction, int32_t aTimeout,
-      const mozilla::dom::Sequence<JS::Value>& aArguments, bool aIsInterval,
-      mozilla::ErrorResult& aError);
-
-  MOZ_CAN_RUN_SCRIPT
-  int32_t SetTimeoutOrInterval(JSContext* aCx, const nsAString& aHandler,
-                               int32_t aTimeout, bool aIsInterval,
-                               mozilla::ErrorResult& aError);
+      JSContext* aCx,
+      const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
+      int32_t aTimeout, const mozilla::dom::Sequence<JS::Value>& aArguments,
+      bool aIsInterval, mozilla::ErrorResult& aError);
 
   // Return true if |aTimeout| was cleared while its handler ran.
   MOZ_CAN_RUN_SCRIPT
-  bool RunTimeoutHandler(mozilla::dom::Timeout* aTimeout,
-                         nsIScriptContext* aScx);
+  bool RunTimeoutHandler(mozilla::dom::Timeout* aTimeout) override;
 
   // Helper Functions
   already_AddRefed<nsIDocShellTreeOwner> GetTreeOwner();
@@ -1184,6 +1173,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   friend class nsPIDOMWindowOuter;
 
   bool IsBackgroundInternal() const override;
+  bool IsPlayingAudio() override;
 
   // NOTE: Chrome Only
   void DisconnectAndClearGroupMessageManagers() {
