@@ -33,6 +33,7 @@ class DOMRect;
 class DOMRectList;
 class InspectorFontFace;
 class Selection;
+class TrustedHTMLOrString;
 
 enum class RangeBehaviour : uint8_t {
   // Keep both ranges
@@ -213,6 +214,9 @@ class nsRange final : public mozilla::dom::AbstractRange,
 
   already_AddRefed<mozilla::dom::DocumentFragment> CreateContextualFragment(
       const nsAString& aString, ErrorResult& aError) const;
+  MOZ_CAN_RUN_SCRIPT already_AddRefed<mozilla::dom::DocumentFragment>
+  CreateContextualFragment(const mozilla::dom::TrustedHTMLOrString&,
+                           ErrorResult& aError) const;
   already_AddRefed<mozilla::dom::DocumentFragment> CloneContents(
       ErrorResult& aErr);
   int16_t CompareBoundaryPoints(uint16_t aHow, const nsRange& aOtherRange,
